@@ -32,6 +32,7 @@
 typedef struct
     {
 	gint 	port_no;
+	gchar 	*port;
 	gchar 	*translation_table;
 	gchar  	*braille_style;
 	gchar 	*cursor_style;
@@ -61,6 +62,7 @@ void 		brlconf_attributes_set 		(gint dots);
 void 		brlconf_device_set 		(const gchar *device);
 void 		brlconf_fill_char_set 		(const gchar *texture);
 void 		brlconf_port_no_set 		(gint port_no);
+void 		brlconf_port_set 		(const gchar *port);
 void 		brlconf_optical_sensor_set 	(gint sensor);
 void 		brlconf_position_sensor_set 	(gint sensor);
 void 		brlconf_status_cell_set 	(const gchar *status);
@@ -75,6 +77,7 @@ gint 		brlconf_attributes_get 		(void);
 gchar* 		brlconf_device_get 		(void);
 gchar* 		brlconf_fill_char_get 		(void);
 gint 		brlconf_port_no_get 		(void);
+gchar*		brlconf_port_get 		(void);
 gint 		brlconf_optical_sensor_get 	(void);
 gint 		brlconf_position_sensor_get 	(void);
 gchar* 		brlconf_status_cell_get 	(void);
