@@ -1,7 +1,7 @@
 /* brlui.c
  *
- * Copyright 2001, 2002 Sun Microsystems, Inc.,
- * Copyright 2001, 2002 BAUM Retec, A.G.
+ * Copyright 2001 - 2005 Sun Microsystems, Inc.,
+ * Copyright 2001 - 2005 BAUM Retec, A.G.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -107,9 +107,10 @@ GtkWidget		*w_braille_settings = NULL;
 **/
 GtkWidget		*w_braille_device;
 static GtkWidget 	*cb_braille_device;
-static GtkWidget 	*sp_port;
+static GtkWidget 	*cb_port;
 static GtkWidget	*lb_port;
-
+static GtkWidget	*ck_man_port;
+static GtkWidget 	*et_man_port;
 /**
  * Translation Table widgets 					
 **/
@@ -310,6 +311,7 @@ brlui_braille_device_changeing (Braille *braille_setting)
 {
     gchar *tmp = NULL;
     gchar *id = NULL;
+    GtkTreeIter iter;
     
     sru_return_if_fail (braille_setting);    
     
@@ -318,9 +320,22 @@ brlui_braille_device_changeing (Braille *braille_setting)
     id  = (gchar*)brlconf_device_id_return (tmp);	 
     if (id)
 	brlconf_device_set (id);
-    braille_setting->port_no = 
-	gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (sp_port));
+
+    if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (cb_port), &iter))
+    {
+	gchar *str, *error;
+	gtk_tree_model_get (gtk_combo_box_get_model (GTK_COMBO_BOX (cb_port)),
+			    &iter, 0, &str, -1);
+	braille_setting->port_no = strtol (str, &error, 10);
+	if ((error && *error) || braille_setting->port_no < MIN_SERIAL_PORT_NO ||
+		braille_setting->port_no > MAX_SERIAL_PORT_NO)
+	    braille_setting->port_no = -1;
+	g_free (str);
+    }
+    else
+        braille_setting->port_no = -1;
     brlconf_port_no_set (braille_setting->port_no);
+    brlconf_port_set (braille_setting->port);
 }
 
 static void
@@ -332,15 +347,28 @@ brlui_braille_device_combo_changed (GtkWidget *entry,
     gdouble min_val, max_val;
 #endif 
     gboolean 	sensitivity;
+    gboolean	active_port_ck, active_port_et, active_port_no;
     sru_return_if_fail (braille_setting);    
     
     if (!strcmp (BRLTTY, gtk_entry_get_text (GTK_ENTRY (entry))))
 	sensitivity = FALSE;
     else
 	sensitivity = TRUE;
-	
-    gtk_widget_set_sensitive (GTK_WIDGET (lb_port), sensitivity);
-    gtk_widget_set_sensitive (GTK_WIDGET (sp_port), sensitivity);
+
+    active_port_ck = active_port_et = active_port_no = FALSE;
+    if (sensitivity)
+    {
+	active_port_ck = TRUE;
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ck_man_port)))
+	    active_port_et = TRUE;
+	else
+	    active_port_no = TRUE;
+    }
+
+    gtk_widget_set_sensitive (GTK_WIDGET (lb_port), active_port_no);
+    gtk_widget_set_sensitive (GTK_WIDGET (cb_port), active_port_no);
+    gtk_widget_set_sensitive (GTK_WIDGET (et_man_port), active_port_et);
+    gtk_widget_set_sensitive (GTK_WIDGET (ck_man_port), active_port_ck);
     
 #if 0    /* remove when it need to select the tty number */
     if (!strcmp (BRLTTY, gtk_entry_get_text (GTK_ENTRY (entry))))
@@ -392,6 +420,35 @@ brlui_delete_emit_response_cancel (GtkDialog *dialog,
     return TRUE; /* Do not destroy */
 }
 
+static void
+brlui_braille_device_ck_man_port_toggled (GtkToggleButton *button, 
+					  gpointer user_data)
+{
+    if (GTK_WIDGET_SENSITIVE (button))
+    {
+	gboolean state = gtk_toggle_button_get_active (button);
+
+	gtk_widget_set_sensitive (et_man_port, state);
+	gtk_widget_set_sensitive (lb_port, !state);
+	gtk_widget_set_sensitive (cb_port, !state);
+	if (!state)
+	{
+	    g_free (braille_setting->port);
+	    braille_setting->port = g_strdup ("");
+	    gtk_entry_set_text (GTK_ENTRY (et_man_port), "");
+	}
+    }
+}
+static gboolean
+brlui_braille_device_et_man_port_changed (GtkWidget *widget,
+                                            GdkEventFocus *event,
+                                            gpointer user_data)
+{
+    g_free (braille_setting->port);
+    braille_setting->port = g_strdup (gtk_entry_get_text (GTK_ENTRY (et_man_port)));
+
+    return FALSE;
+}
 
 void 
 brlui_set_handlers_braille_device (GladeXML *xml,
@@ -399,8 +456,10 @@ brlui_set_handlers_braille_device (GladeXML *xml,
 {
     w_braille_device  = glade_xml_get_widget (xml, "w_braille_device");
     cb_braille_device = glade_xml_get_widget (xml, "cb_braille_device");
-    sp_port 	      = glade_xml_get_widget (xml, "sp_port");
+    cb_port 	      = glade_xml_get_widget (xml, "cb_port");
     lb_port 	      = glade_xml_get_widget (xml, "lb_port");
+    ck_man_port	      = glade_xml_get_widget (xml, "ck_man_port");
+    et_man_port	      = glade_xml_get_widget (xml, "et_man_port");
 
     g_signal_connect (w_braille_device, "response",
 		      G_CALLBACK (brlui_settings_response_braille_device), 
@@ -410,6 +469,12 @@ brlui_set_handlers_braille_device (GladeXML *xml,
 
     glade_xml_signal_connect (xml,"on_combo-entry1_changed",		
 			    GTK_SIGNAL_FUNC (brlui_braille_device_combo_changed));
+			    
+    glade_xml_signal_connect (xml,"on_ck_man_port_toggled",		
+			    GTK_SIGNAL_FUNC (brlui_braille_device_ck_man_port_toggled));
+
+    glade_xml_signal_connect (xml,"on_et_man_port_focus_out_event",		
+			    GTK_SIGNAL_FUNC (brlui_braille_device_et_man_port_changed));
 
     gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (cb_braille_device)->entry), FALSE);
 }
@@ -421,6 +486,7 @@ brlui_load_braille_device (Braille *braille_setting)
     if (!w_braille_device)
     {
 	GladeXML *xml;
+	gint i;
 	xml = gn_load_interface ("Braille_Settings/braille_device.glade2", NULL);
 	sru_return_val_if_fail (xml, FALSE);
     	brlui_set_handlers_braille_device (xml, braille_setting);
@@ -429,6 +495,14 @@ brlui_load_braille_device (Braille *braille_setting)
 				      GTK_WINDOW (w_braille_settings));
 	gtk_window_set_destroy_with_parent (GTK_WINDOW (w_braille_device), 
 						TRUE);
+	gtk_combo_box_append_text (GTK_COMBO_BOX (cb_port), _("<none>"));
+	for (i = 0; i <= MAX_SERIAL_PORT_NO; i++)
+	{
+	    gchar *port;
+	    port = g_strdup_printf ("%d", i);
+	    gtk_combo_box_append_text (GTK_COMBO_BOX (cb_port), port);
+	    g_free (port);
+	}
     }
     else
 	gtk_widget_show (w_braille_device);
@@ -451,6 +525,13 @@ brlui_braille_device_set_to_combo (GList *brldev_list)
 gboolean 
 brlui_braille_device_value_add_to_widgets (Braille *braille_setting)
 {    
+    GtkTreeModel* model;
+    GtkTreeIter iter, iter_found;
+    gboolean found = FALSE, hasnone = FALSE;
+    gint port;
+    gchar *str, *error;
+    const gchar *dev;
+
     sru_return_val_if_fail (braille_setting, FALSE);    
     
     if (!w_braille_device)
@@ -460,10 +541,47 @@ brlui_braille_device_value_add_to_widgets (Braille *braille_setting)
     
     brlui_braille_device_set_to_combo (brldev_description_list);
 
+    dev = brlconf_return_active_device ();
+    if (!dev)
+	dev = _("<none>");
     gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (cb_braille_device)->entry), 
-			brlconf_return_active_device ());
-    gtk_spin_button_set_value (GTK_SPIN_BUTTON (sp_port), 
-			       (gdouble)braille_setting->port_no);
+			dev);
+
+    model = gtk_combo_box_get_model (GTK_COMBO_BOX (cb_port));
+    if (!model)
+	return FALSE;
+    if (!gtk_tree_model_get_iter_first (model, &iter))
+	return FALSE;
+
+    while (!found)
+    {
+	gtk_tree_model_get (model, &iter, 0, &str, -1);
+	if (strcmp (str, _("<none>")) == 0 && !found)
+	{
+	    iter_found = iter;
+	    hasnone= TRUE;
+	}
+	port = strtol (str, &error, 10);
+	if ((!error || !*error) && port == braille_setting->port_no)
+	{
+	    sru_assert (found == FALSE);
+	    iter_found = iter;
+	    found = TRUE;
+	}
+	g_free (str);
+	if (!gtk_tree_model_iter_next (model, &iter))
+	    break;
+    };
+    if (found || hasnone)
+	gtk_combo_box_set_active_iter (GTK_COMBO_BOX (cb_port), &iter_found);
+    gtk_entry_set_text (GTK_ENTRY (et_man_port), braille_setting->port);
+    if (braille_setting->port && braille_setting->port[0])
+    {
+	gtk_widget_set_sensitive (GTK_WIDGET (lb_port), FALSE);
+	gtk_widget_set_sensitive (GTK_WIDGET (cb_port), FALSE);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ck_man_port), TRUE);
+    }
+
     return TRUE;
 }
 
